'use strict';

/**
 * Created by yeobster on 15. 3. 5..
 */

const DiskAdapter = require('sails-disk'),
    MemoryAdapter = require('sails-memory'),
    MongoAdapter = require('sails-mongo'),
    MysqlAdapter = require('sails-mysql'),
    PostgresqlAdapter = require('sails-postgresql'),
    RedisAdapter = require('sails-redis'),
    SqlserverAdapter = require('sails-sqlserver'),
    Waterline = require('waterline'),
    fs = require('fs'),
    _ = require('lodash');

/**
 * connections
 * @desc Configuration of database adapters
 * @author yeobster@gmail.com
 */

/******************************************
 *  # mysql connection
 *
 *  3. set connection with attributes
 *  adapter: 'mysql',
 *  host      : 'localhost',
 *  port      : 3306,
 *  user      : 'username',
 *  password  : 'password',
 *  database  : 'MySQL Database Name'
 *
 *  // Optional
 *  charset   : 'utf8',
 *  collation : 'utf8_swedish_ci'
 *
 *  pool: true,
 *  connectionLimit: 10,
 *  waitForConnections: true
 ******************************************/

function connections(config, migrate) {
    const adapterList = {
        'disk': DiskAdapter,
        'memory': MemoryAdapter,
        'mongo': MongoAdapter,
        'mysql': MysqlAdapter,
        'postgresql': PostgresqlAdapter,
        'redis': RedisAdapter,
        'sqlserver': SqlserverAdapter
    };

    let adapters = {};
    let datastores = {};
    for (let db in config) {
        adapters[db] = adapterList[db];
        datastores[db + 'Connection'] = {
            adapter: db,
            host: config[db].host,
            port: config[db].port,
            user: config[db].username,
            password: config[db].password,
            database: config[db].database
        };
    }

    return {
        adapters: adapters,
        datastores: datastores,
        defaults: {
            migrate: migrate //drop, safe, create, alter
        }
    };
}


/**
 * db connection log
 * @param connections
 */
function connectionLog(connections) {
    // db connection log
    for (let key in connections) {
        let connection = connections[key];
        if (connection.config) {
            let adapter = '\x1b[35m' + connection.config.adapter + '\x1b[0m';
            let server = ' \x1b[1m\x1b[33m//' + connection.config.host + ':' + connection.config.port + '/' + connection.config.database + '\x1b[0m';
            console.info('\x1b[32m>>\x1b[0m', 'DB connected ' + adapter + server);
        }
    }
};

/**
 * initialize models
 * @param modelPath
 * @returns {Array}
 */
function initModel(modelPath) {
    let models = [];
    if (fs.existsSync(modelPath)) {
        let files = fs.readdirSync(modelPath);
        for (let i = 0; i < files.length; i++) {
            let filename = modelPath + '/' + files[i];
            models.push(require(filename));
        }
    }
    return models;
}

exports.register = (server, options) => {
    if (!options.config) {
        throw new Error('ERR : no database configuration');
    }
    if (!options.modelPath) {
        throw new Error('ERR : no model path');
    }

    let models = initModel(options.modelPath);

    // Instantiate a new instance of the ORM
    let orm = new Waterline();

    if (models.length > 0) {
        // Load the Models into the ORM
        for (let i = 0; i < models.length; i++) {
            let model = models[i];
            orm.loadCollection(Waterline.Collection.extend(model));
        }

        orm.initialize(connections(options.config, options.migrate), function (err, model) {
            if (err) {
                throw err;
            }

            // Expose collections to server
            server.expose('models', model.collections);

            // Create global letiable for models
            if (options.useGlobalModel) {
                for (let index in model.collections) {
                    global[_.upperFirst(index)] = model.collections[index];
                }
            }

            // db connection log
            connectionLog(model.connections);

            // Allow servers to access the raw ORM!
            server.method('getWaterline', function (orm) {
                return function (cb) {
                    return cb(null, orm);
                };
            }(orm));
        });
    }    
};

exports.pkg = require('../package.json');